/*  smplayer, GUI front-end for mplayer.
    Copyright (C) 2007 Ricardo Villalba <rvm@escomposlinux.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "preferencesdialog.h"
#include "helper.h"
#include "images.h"
#include "global.h"
#include "encodings.h"

#include <qlistbox.h>
#include <qwidgetstack.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qpushbutton.h>
#include <qcombobox.h>
#include <qspinbox.h>
#include <qcheckbox.h>
#include <qcolor.h>
#include <qslider.h>
#include <qfontdialog.h>
#include <qfiledialog.h>
#include <qcolordialog.h>
#include <qfileinfo.h>
#include <qdir.h>
#include <qstylefactory.h>
#include <qlayout.h> // QSpacerItem

#define SECTION_GENERAL 0
#define SECTION_DRIVES 1
#define SECTION_PERFORMANCE 2
#define SECTION_SUBTITLES 3
#define SECTION_MISC 4
#define SECTION_ADVANCED 5

#define PAGE_GENERAL 0
#define PAGE_SUBTITLES 1
#define PAGE_ADVANCED 2
#define PAGE_PERFORMANCE 3
#define PAGE_MISC 4
#define PAGE_DRIVES 5

PreferencesDialog::PreferencesDialog( QWidget* parent, const char* name, bool modal, WFlags fl )
	: PreferencesDialogBase(parent, name, modal, fl)
{
    sections->insertItem( images->icon("pref_general"),
                          tr( "General" ), SECTION_GENERAL );
    sections->insertItem( images->icon("pref_devices"),
                          tr( "Drives" ), SECTION_DRIVES );
    sections->insertItem( images->icon("pref_performance"),
                          tr( "Performance" ), SECTION_PERFORMANCE );
    sections->insertItem( images->icon("pref_subtitles"), 
                          tr( "Subtitles" ), SECTION_SUBTITLES );
    sections->insertItem( images->icon("pref_advanced"),
                          tr( "Advanced" ), SECTION_ADVANCED );
    sections->insertItem( images->icon("pref_misc"),
                          tr( "Misc" ), SECTION_MISC );

	connect( sections, SIGNAL(highlighted(int)), this, SLOT(changeSection(int)) );

	connect( search_mplayerbin_button, SIGNAL(clicked()),
             this, SLOT(selectMplayerPath()) );

	connect( screenshot_button, SIGNAL(clicked()),
             this, SLOT(selectScreenshotDirectory()) );

	connect( remember_all_check, SIGNAL(toggled(bool)),
             this, SLOT(rememberAllButtonToggled(bool)) );

	connect( system_font_button, SIGNAL(clicked()),
             this, SLOT(systemFontRadioButtonSelected()) );

	connect( ttf_font_button, SIGNAL(clicked()),
             this, SLOT(ttfFontRadioButtonSelected()) );

	connect( select_system_font_button, SIGNAL(clicked()),
             this, SLOT(selectSystemFont()) );

	connect( select_ttf_font_button, SIGNAL(clicked()),
             this, SLOT(selectTtfFont()) );

	connect( font_ass_check, SIGNAL(toggled(bool)),
             this, SLOT(assButtonToggled(bool)) );

	connect( asscolor_button, SIGNAL(clicked()),
             this, SLOT(selectAssColor()) );

	connect( assbordercolor_button, SIGNAL(clicked()),
             this, SLOT(selectAssBorderColor()) );

	connect( autosync_check, SIGNAL(toggled(bool)),
             this, SLOT(autoSyncButtonToggled(bool)) );

	sections->setSelected( SECTION_GENERAL, TRUE );

	// Style combo
#if STYLE_SWITCHING
	style_combo->insertItem( "<default>" );
	QStringList k = QStyleFactory::keys();
	for ( QStringList::Iterator it = k.begin(); it != k.end(); ++it ) {
		style_combo->insertItem( *it );
    }
#endif

	// DVD device combo
	// In windows, insert the drives letters
#ifdef Q_OS_WIN
	#if QT_VERSION < 0x040000
	const QFileInfoList* roots = QDir::drives();
    QPtrListIterator<QFileInfo> i(*roots);
    QFileInfo* fi;
    while ( (fi = *i) ) {
        ++i;
		QString s = fi->filePath();
		if (s.endsWith("/")) s = s.remove( s.length()-1,1);
		dvd_device_combo->insertItem( s );
        cdrom_device_combo->insertItem( s );
    }
	#else
	QFileInfoList list = QDir::drives();
	for (int n = 0; n < list.size(); n++) {
		QString s = list[n].filePath();
		if (s.endsWith("/")) s = s.remove( s.length()-1,1);
		dvd_device_combo->insertItem( s );
		cdrom_device_combo->insertItem( s );
	}
	#endif
#else
#define ADD_IF_EXISTS( string ) \
	if (QFileInfo( string ).exists()) { \
		dvd_device_combo->insertItem( string ); \
        cdrom_device_combo->insertItem( string ); \
    }

	ADD_IF_EXISTS("/dev/dvd");
	ADD_IF_EXISTS("/dev/dvdrecorder");
	ADD_IF_EXISTS("/dev/cdrom");
	ADD_IF_EXISTS("/dev/cdrecorder");
#endif

	// Priority is only for windows, so we disable for other systems
#ifndef Q_OS_WIN
	priority_label->setEnabled(FALSE);
	priority_combo->setEnabled(FALSE);
	priority_intro_label->setText( priority_intro_label->text() + "\n" +
         tr("(This option is for Windows only)") );
	/*
	priority_label->hide();
	priority_combo->hide();
	priority_intro_label->hide();
	priority_separator->hide();
	priority_spacer->changeSize(0,0);
	*/
#endif

	drivers_set = FALSE;

	// Monitor aspect
	monitoraspect_combo->insertItem("Auto");
	monitoraspect_combo->insertItem("4:3");
	monitoraspect_combo->insertItem("16:9");
	monitoraspect_combo->insertItem("5:4");
	monitoraspect_combo->insertItem("16:10");

#if STYLE_SWITCHING == 0
	style_label->hide();
	style_combo->hide();
#endif

	autoq_slider->setTickmarks( QSlider::Below );
	autoq_slider->setTickInterval( 1 );
	autoq_slider->setLineStep( 1 );
	autoq_slider->setPageStep( 1 );

	encodings = new Encodings(this, "encodings" );
	font_encoding_combo->insertStringList( encodings->list() );

	setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );
	adjustSize();
}

PreferencesDialog::~PreferencesDialog() {
}

void PreferencesDialog::setDrivers(InfoList vo_list, 
                                   InfoList ao_list)
{
	InfoList::iterator it;

	for ( it = vo_list.begin(); it != vo_list.end(); ++it ) {
		vo_combo->insertItem( (*it).name() );
		// Add directx:noaccel
		if ( (*it).name() == "directx" ) {
			vo_combo->insertItem( "directx:noaccel" );
		}
	}

	for ( it = ao_list.begin(); it != ao_list.end(); ++it ) {
		ao_combo->insertItem( (*it).name() );
	}

	drivers_set = TRUE;
}

void PreferencesDialog::accept() {
	hide();
	setResult( QDialog::Accepted );
	emit applyButtonPressed();
}

void PreferencesDialog::apply() {
	setResult( QDialog::Accepted );
	emit applyButtonPressed();
}


void PreferencesDialog::changeSection(int n) {
	qDebug("PreferencesDialog::changeSection: %d", n);

	switch (n) {
		case SECTION_SUBTITLES:
			pages->raiseWidget( PAGE_SUBTITLES );
			break;
		case SECTION_ADVANCED:
			pages->raiseWidget( PAGE_ADVANCED );
			break;
		case SECTION_PERFORMANCE:
			pages->raiseWidget( PAGE_PERFORMANCE );
			break;
		case SECTION_MISC:
			pages->raiseWidget( PAGE_MISC );
			break;
		case SECTION_DRIVES:
			pages->raiseWidget( PAGE_DRIVES );
			break;
		default:
			pages->raiseWidget( PAGE_GENERAL );
	}
}



// General Page
void PreferencesDialog::setMplayerPath( QString path ) {
	mplayerbin_edit->setText( path );	
}

QString PreferencesDialog::mplayerPath() {
	return mplayerbin_edit->text();
}

void PreferencesDialog::setScreenshotDir( QString path ) {
	screenshot_edit->setText( path );
}

QString PreferencesDialog::screenshotDir() {
	return screenshot_edit->text();
}

void PreferencesDialog::setVO( QString vo_driver ) {
	vo_combo->setCurrentText( vo_driver );
}

void PreferencesDialog::setAO( QString ao_driver ) {
	ao_combo->setCurrentText( ao_driver );
}

QString PreferencesDialog::VO() {
	return vo_combo->currentText();
}

QString PreferencesDialog::AO() {
	return ao_combo->currentText();
}

void PreferencesDialog::setEq2(bool b) {
	eq2_check->setChecked(b);
}

bool PreferencesDialog::eq2() {
	return eq2_check->isChecked();
}

void PreferencesDialog::setSoftVol(bool b) {
	softvol_check->setChecked(b);
}

bool PreferencesDialog::softVol() {
	return softvol_check->isChecked();
}

void PreferencesDialog::setRememberSettings(bool b) {
	remember_all_check->setChecked(b);
	rememberAllButtonToggled(b);
}

bool PreferencesDialog::rememberSettings() {
	return remember_all_check->isChecked();
}

void PreferencesDialog::setDontRememberTimePos(bool b) {
	dont_remember_time_check->setChecked(b);
}

bool PreferencesDialog::dontRememberTimePos() {
	return dont_remember_time_check->isChecked();
}

void PreferencesDialog::setStartInFullscreen(bool b) {
	start_fullscreen_check->setChecked(b);
}

bool PreferencesDialog::startInFullscreen() {
	return start_fullscreen_check->isChecked();
}

void PreferencesDialog::setSubtitlesOnScreenshots(bool b) {
	subtitles_on_screeshots_check->setChecked(b);
}

bool PreferencesDialog::subtitlesOnScreenshots() {
	return subtitles_on_screeshots_check->isChecked();
}


// Drive page
void PreferencesDialog::setDVDDevice( QString dir ) {
	dvd_device_combo->setCurrentText( dir );
}

QString PreferencesDialog::dvdDevice() {
	return dvd_device_combo->currentText();
}

void PreferencesDialog::setCDRomDevice( QString dir ) {
	cdrom_device_combo->setCurrentText( dir );
}

QString PreferencesDialog::cdromDevice() {
	return cdrom_device_combo->currentText();
}

// Misc Page
void PreferencesDialog::setDisableScreensaver(bool b) {
	screensaver_check->setChecked(b);
}

bool PreferencesDialog::disableScreensaver() {
	return screensaver_check->isChecked();
}

void PreferencesDialog::setResizeMethod(int v) {
	mainwindow_resize_combo->setCurrentItem(v);
}

int PreferencesDialog::resizeMethod() {
	return mainwindow_resize_combo->currentItem();
}

void PreferencesDialog::setWheelSeek(int secs) {
	wheel_seek_spin->setValue(secs);
}

int PreferencesDialog::wheelSeek() {
	return wheel_seek_spin->value();
}

void PreferencesDialog::setMonitorAspect(QString asp) {
	if (asp.isEmpty())
		monitoraspect_combo->setCurrentText("Auto");
	else
		monitoraspect_combo->setCurrentText(asp);
}

QString PreferencesDialog::monitorAspect() {
	if (monitoraspect_combo->currentText() == "Auto") 
		return "";
	else
		return monitoraspect_combo->currentText();
}

void PreferencesDialog::setStyle(QString style) {
	if (style.isEmpty()) style="<default>";
	style_combo->setCurrentText(style);
}

QString PreferencesDialog::style() {
	QString s = style_combo->currentText();
	if (s=="<default>") s="";
	return s;
}


// General Page stuff
void PreferencesDialog::selectMplayerPath() {
	QString	s = QFileDialog::getOpenFileName(
	                    mplayerbin_edit->text(),
#ifdef Q_OS_WIN
    	                tr("Executables") +" (*.exe)",
#else
    	                tr("All files") +" (*)",
#endif
        	            this, "select_mplayer_dialog",
            	        tr("Select the mplayer executable") );

	if (!s.isEmpty()) {
		mplayerbin_edit->setText(s);
	}
}


void PreferencesDialog::selectScreenshotDirectory() {
	QString s = QFileDialog::getExistingDirectory(
                    screenshot_edit->text(),
                    this,
                    "select_screenshot_directory",
                    tr("Select a directory"),
                    TRUE );
	if (!s.isEmpty()) {
		screenshot_edit->setText(s);
	}
}


void PreferencesDialog::rememberAllButtonToggled(bool b) {
	dont_remember_time_check->setEnabled(b);
}


// Subtitles Page
void PreferencesDialog::setFontName(QString font_name) {
	system_font_label->setText( font_name );
}

void PreferencesDialog::setFontFile(QString font_file) {
	ttf_font_edit->setText( font_file );
}

void PreferencesDialog::setUseFontconfig(bool b) {
	if (b) {
		system_font_button->setChecked(TRUE);
		ttf_font_button->setChecked(FALSE);
		systemFontRadioButtonSelected();
	} else {
		ttf_font_button->setChecked(TRUE);
		system_font_button->setChecked(FALSE);
		ttfFontRadioButtonSelected();
	}
}

QString PreferencesDialog::fontName() {
	return system_font_label->text();
}

QString PreferencesDialog::fontFile() {
	return ttf_font_edit->text();
}

bool PreferencesDialog::useFontconfig() {
	return system_font_button->isChecked();
}

void PreferencesDialog::setFontAutoscale(int n) {
	font_autoscale_combo->setCurrentItem(n);
}

int PreferencesDialog::fontAutoscale() {
	return font_autoscale_combo->currentItem();
}

void PreferencesDialog::setFontTextscale(int n) {
	font_text_scale->setValue(n);
}

int PreferencesDialog::fontTextscale() {
	return font_text_scale->value();
}

void PreferencesDialog::setAutoloadSub(bool v) {
	font_autoload_check->setChecked(v);
}

bool PreferencesDialog::autoloadSub() {
	return font_autoload_check->isChecked();
}

void PreferencesDialog::setFontEncoding(QString s) {
	int n = encodings->findEncoding( s );
	if (n != -1) 
		font_encoding_combo->setCurrentItem(n);
	else
		font_encoding_combo->setCurrentText(s);
}

QString PreferencesDialog::fontEncoding() {
	qDebug("PreferencesDialog::fontEncoding");
	QString res = encodings->parseEncoding( font_encoding_combo->currentText() );
	qDebug(" * res: '%s'", res.utf8().data() );
	return res;
}

void PreferencesDialog::setUseFontASS(bool v) {
	font_ass_check->setChecked(v);
	assButtonToggled(v);
}

bool PreferencesDialog::useFontASS() {
	return font_ass_check->isChecked();
}

void PreferencesDialog::setAssColor( unsigned int color ) {
	ass_color = color;
	#if QT_VERSION >= 0x040200
	asscolor_button->setStyleSheet( "border-width: 1px; border-style: solid; border-color: #000000; background: #" + Helper::colorToRGB(ass_color) + ";");
	#else
	asscolor_button->setPaletteBackgroundColor( ass_color );
	#endif
}

unsigned int PreferencesDialog::assColor() {
	return ass_color;
}

void PreferencesDialog::setAssBorderColor( unsigned int color ) {
	ass_border_color = color;

	#if QT_VERSION >= 0x040200
	assbordercolor_button->setStyleSheet( "border-width: 1px; border-style: solid; border-color: #000000; background: #" + Helper::colorToRGB(ass_border_color) + ";");
	#else
	assbordercolor_button->setPaletteBackgroundColor( ass_border_color );
	#endif
}

unsigned int PreferencesDialog::assBorderColor() {
	return ass_border_color;
}

void PreferencesDialog::setFontFuzziness(int n) {
	font_autoload_combo->setCurrentItem(n);
}

int PreferencesDialog::fontFuzziness() {
	return font_autoload_combo->currentItem();
}

// Subtitles Page stuff
void PreferencesDialog::systemFontRadioButtonSelected() {
	qDebug("PreferencesDialog::systemFontRadioButtonSelected");

	system_font_label->setEnabled(TRUE);
	select_system_font_button->setEnabled(TRUE);

	ttf_font_edit->setEnabled(FALSE);
	select_ttf_font_button->setEnabled(FALSE);

}

void PreferencesDialog::ttfFontRadioButtonSelected() {
	qDebug("PreferencesDialog::ttfFontRadioButtonSelected");

	ttf_font_edit->setEnabled(TRUE);
	select_ttf_font_button->setEnabled(TRUE);

	system_font_label->setEnabled(FALSE);
	select_system_font_button->setEnabled(FALSE);
}

void PreferencesDialog::assButtonToggled(bool b) {
	asscolor_label->setEnabled(b);
	asscolor_button->setEnabled(b);
	assbordercolor_label->setEnabled(b);
	assbordercolor_button->setEnabled(b);
}

void PreferencesDialog::selectSystemFont() {
	bool ok;
    QFont font = QFontDialog::getFont(
                    &ok, QFont( system_font_label->text(), 10 ), this );
    if ( ok ) {
        // font is set to the font the user selected
		system_font_label->setText( font.family() );
    } else {
        // the user canceled the dialog; font is set to the initial
        // value, in this case Helvetica [Cronyx], 10
    }

}

void PreferencesDialog::selectTtfFont() {
	QString s = "";

#ifdef Q_OS_WIN
	// The windows native file dialog doesn't allow to select a ttf file! :-O
	// So we use a Qt one.
	QFileDialog fd( QFileInfo(ttf_font_edit->text()).dirPath(TRUE),
                    tr("Truetype Fonts") +" (*.ttf)",
                    this, "open file dialog", TRUE );

	if ( fd.exec() == QDialog::Accepted ) {
		s = fd.selectedFile();
	}
#else
	s = QFileDialog::getOpenFileName(
	                    ttf_font_edit->text(),
    	                tr("Truetype Fonts") + " (*.ttf)",
        	            this, "open file dialog",
            	        tr("Choose a ttf file") );
#endif
	if (!s.isEmpty()) {
		ttf_font_edit->setText(s);
	}
}

void PreferencesDialog::selectAssColor() {
	QColor c = QColorDialog::getColor ( ass_color, this );
	if (c.isValid()) {
		setAssColor( c.rgb() );
	}
}

void PreferencesDialog::selectAssBorderColor() {
	QColor c = QColorDialog::getColor ( ass_border_color, this );
	if (c.isValid()) {
		setAssBorderColor( c.rgb() );
	}
}

// MPlayer Page
void PreferencesDialog::setUseMplayerWindow(bool v) {
	mplayer_use_window_check->setChecked(v);
}

bool PreferencesDialog::useMplayerWindow() {
	return mplayer_use_window_check->isChecked();
}

void PreferencesDialog::setCache(int n) {
	cache_spin->setValue(n);
}

int PreferencesDialog::cache() {
	return cache_spin->value();
}

void PreferencesDialog::setMplayerAdditionalArguments(QString args) {
	mplayer_args_edit->setText(args);
}

QString PreferencesDialog::mplayerAdditionalArguments() {
	return mplayer_args_edit->text();
}

void PreferencesDialog::setMplayerAdditionalVideoFilters(QString s) {
	mplayer_vfilters_edit->setText(s);
}

QString PreferencesDialog::mplayerAdditionalVideoFilters() {
	return mplayer_vfilters_edit->text();
}

void PreferencesDialog::setMplayerAdditionalAudioFilters(QString s) {
	mplayer_afilters_edit->setText(s);
}

QString PreferencesDialog::mplayerAdditionalAudioFilters() {
	return mplayer_afilters_edit->text();
}

// Performance Page
void PreferencesDialog::setPriority(int n) {
	priority_combo->setCurrentItem(n);
}

int PreferencesDialog::priority() {
	return priority_combo->currentItem();
}

void PreferencesDialog::setFrameDrop(bool b) {
	framedrop_check->setChecked(b);
}

bool PreferencesDialog::frameDrop() {
	return framedrop_check->isChecked();
}

void PreferencesDialog::setHardFrameDrop(bool b) {
	hardframedrop_check->setChecked(b);
}

bool PreferencesDialog::hardFrameDrop() {
	return hardframedrop_check->isChecked();
}


void PreferencesDialog::setAutoq(int n) {
	autoq_slider->setValue(n);
}

int PreferencesDialog::autoq() {
	return autoq_slider->value();
}

void PreferencesDialog::setAutoSyncFactor(int factor) {
	autosync_spin->setValue(factor);
}

int PreferencesDialog::autoSyncFactor() {
	return autosync_spin->value();
}

void PreferencesDialog::setAutoSyncActivated(bool b) {
	autosync_check->setChecked(b);
	autoSyncButtonToggled(b);
}

bool PreferencesDialog::autoSyncActivated() {
	return autosync_check->isChecked();
}

void PreferencesDialog::autoSyncButtonToggled(bool b) {
	factor_label->setEnabled(b);
	autosync_spin->setEnabled(b);
}

void PreferencesDialog::setFastChapterSeeking(bool b) {
	fast_chapter_check->setChecked(b);
}

bool PreferencesDialog::fastChapterSeeking() {
	return fast_chapter_check->isChecked();
}

void PreferencesDialog::setFastAudioSwitching(bool b) {
	fastaudioswitching_check->setChecked(b);
}

bool PreferencesDialog::fastAudioSwitching() {
	return fastaudioswitching_check->isChecked();
}

